(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('axios'), require('file-type'), require('fs'), require('node:crypto')) :
  typeof define === 'function' && define.amd ? define(['exports', 'axios', 'file-type', 'fs', 'node:crypto'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.BunnyCdnStream = {}, global.axios, global.fileType, global.fs, global.crypto));
})(this, (function (exports, axios, fileType, fs, node_crypto) { 'use strict';

  const lowerObject = (obj) => {
      const newObj = {};
      for (const [key, value] of Object.entries(obj)) {
          newObj[key[0].toLowerCase() + key.slice(1)] = value;
      }
      return newObj;
  };

  const errorTypes = {
      404: 'NOT_FOUND',
      400: 'BAD_REQUEST',
      401: 'UNAUTHORIZED',
      403: 'FORBIDDEN',
      500: 'INTERNAL_SERVER_ERROR',
      502: 'BAD_GATEWAY',
      503: 'SERVICE_UNAVAILABLE',
      504: 'GATEWAY_TIMEOUT'
  };
  class BunnyCdnStreamError extends Error {
      constructor(axiosError, when, code) {
          super();
          this.name = 'BunnyCdnStreamError';
          if (axiosError instanceof axios.AxiosError) {
              this.message = `BunnyCdnStreamError: Operation "${when}" - ${axiosError.response ? errorTypes[axiosError.response.status] : 'UNKNOWN_ERROR'} ${axiosError.message}`;
              this.code = axiosError.response ? axiosError.response.status : 0;
              if (axiosError.response?.data) {
                  if (typeof axiosError.response.data === 'object') {
                      const data = lowerObject(axiosError.response.data);
                      if ('error' in data) {
                          this.message += `: ${data.error}`;
                      }
                      if ('message' in data) {
                          this.message += `: ${data.message}`;
                      }
                  }
                  else {
                      this.message += `: ${JSON.stringify(axiosError.response.data)}`;
                  }
              }
          }
          else {
              this.code = code || -1;
              this.message = `BunnyCdnStreamError: Unable to ${when}, ${axiosError}`;
          }
      }
  }

  class BunnyCdnStreamVideo {
      constructor(data) {
          this.videoLibraryId = data.videoLibraryId;
          this.guid = data.guid;
          this.title = data.title;
          this.dateUploaded = data.dateUploaded;
          this.views = data.views;
          this.isPublic = data.isPublic;
          this.length = data.length;
          this.status = data.status;
          this.framerate = data.framerate;
          this.width = data.width;
          this.height = data.height;
          this.availableResolutions = data.availableResolutions;
          this.thumbnailCount = data.thumbnailCount;
          this.encodeProgress = data.encodeProgress;
          this.storageSize = data.storageSize;
          this.captions = data.captions;
          this.hasMP4Fallback = data.hasMP4Fallback;
          this.collectionId = data.collectionId;
          this.thumbnailFileName = data.thumbnailFileName;
          this.moments = data.moments;
          this.captions = data.captions;
          this.averageWatchTime = data.averageWatchTime;
          this.totalWatchTime = data.totalWatchTime;
          this.category = data.category;
          this.chapters = data.chapters;
          this.metaTags = data.metaTags;
      }
      get resolutions() {
          return this.availableResolutions.split(',').map((x) => parseInt(x, 10));
      }
  }

  class BunnyCdnStream {
      constructor(options) {
          this.axiosOptions = {
              headers: new axios.AxiosHeaders({
                  Accept: 'application/json',
                  'Content-Type': 'application/json',
                  AccessKey: ''
              }),
              url: 'https://video.bunnycdn.com',
              method: 'GET',
              maxBodyLength: Infinity
          };
          this.options = options;
          this.axiosOptions.headers.AccessKey = this.options.apiKey;
      }
      /**
       * Retrieve a video from BunnyCdn
       * @returns A {@link BunnyCdnStream.VideoResponse} instance.
       * @param videoId The video ID
       * @example
       * ```typescript
       * await stream.getVideo("0273f24a-79d1-d0fe-97ca-b0e36bed31es")
       * ```
       */
      async getVideo(videoId) {
          const options = this.getOptions();
          options.url += `/library/${this.options.videoLibrary}/videos/${videoId}`;
          const video = await this.request(options, 'fetch');
          return new BunnyCdnStreamVideo(video);
      }
      /**
       * Update video information
       * @returns A {@link BunnyCdnStream.VideoResponse} instance.
       * @param videoId The video ID
       * @param data The data to update
       * @example
       * ```typescript
       * await stream.updateVideo("0273f24a-79d1-d0fe-97ca-b0e36bed31es", { title: "New title" })
       * ```
       */
      async updateVideo(videoId, data = {}) {
          const options = this.getOptions();
          options.url += `/library/${this.options.videoLibrary}/videos/${videoId}`;
          options.method = 'POST';
          options.data = JSON.stringify(data);
          return this.request(options, 'update');
      }
      /**
       * Delete a video
       * @returns A {@link BunnyCdnStream.DeleteVideoResponse} instance.
       * @param videoId The video ID
       * @example
       * ```typescript
       * await stream.deleteVideo("0273f24a-79d1-d0fe-97ca-b0e36bed31es")
       * ```
       */
      async deleteVideo(videoId) {
          const options = this.getOptions();
          options.url += `/library/${this.options.videoLibrary}/videos/${videoId}`;
          options.method = 'DELETE';
          return this.request(options, 'delete');
      }
      /**
       * Delete all videos
       *
       * NOTE: This uses the listVideos method and will iterate over all pages and delete all videos per page before continuing to the next page.
       * @returns void
       * @example
       * ```typescript
       * await stream.deleteAllVideos();
       * ```
       */
      async deleteAllVideos() {
          const iterate = async (page) => {
              const { items: videos } = await this.listVideos({ page });
              if (videos.length === 0)
                  return;
              await Promise.all(videos.map((video) => this.deleteVideo(video.guid)));
              await iterate(page + 1);
          };
          await iterate(1);
      }
      /**
       * Create a video, this does not upload the video file
       * @returns A {@link BunnyCdnStream.VideoResponse} instance.
       * @param data The data to create the video with
       * @example
       * ```typescript
       * await stream.createVideo({ title: "The best title" })
       * ```
       */
      async createVideo(data) {
          const options = this.getOptions();
          options.url += `/library/${this.options.videoLibrary}/videos`;
          options.method = 'POST';
          options.data = JSON.stringify(data);
          console.log(options);
          const video = await this.request(options, 'create');
          return new BunnyCdnStreamVideo(video);
      }
      /**
       * Upload video, this does not create the video and requires a created video
       * @returns A {@link BunnyCdnStream.UploadVideoResponse} instance.
       * @param file The video file to upload as a readable stream
       * @param videoId The video id to upload to of a created video
       * @param data Optional paramaters such as enabledResolutions
       * @example
       * ```typescript
       * await stream.uploadVideo(createReadStream("./file.mp4"), "0273f24a-79d1-d0fe-97ca-b0e36bed31es")
       * ```
       */
      async uploadVideo(file, videoId, data) {
          const options = this.getOptions();
          options.url += `/library/${this.options.videoLibrary}/videos/${videoId}`;
          options.method = 'PUT';
          options.data = file;
          options.params = data;
          options.headers.set('Content-Type', 'application/octet-stream');
          return this.request(options, 'upload');
      }
      /**
       * Create and upload a video in one function
       * @returns A {@link BunnyCdnStream.VideoResponse} instance.
       * @param file The video file to upload as a readable stream
       * @param data The data to create the video with
       * @example
       * ```typescript
       * await stream.createAndUploadVideo(createReadStream("./file.mp4"), { title: "The best title" })
       * ```
       */
      async createAndUploadVideo(file, data) {
          const createdVideo = await this.createVideo(data);
          await this.uploadVideo(file, createdVideo.guid);
          return createdVideo;
      }
      /**
       * Get video statistics
       * @returns A {@link BunnyCdnStream.VideoHeatmapResponse} instance.
       * @param videoId The video id to get heatmap info from
       * @example
       * ```typescript
       * await stream.getVideoHeatmap("0273f24a-79d1-d0fe-97ca-b0e36bed31es")
       * ```
       */
      async getVideoHeatmap(videoId) {
          const options = this.getOptions();
          options.method = 'GET';
          options.url += `/library/${this.options.videoLibrary}/videos/${videoId}/heatmap`;
          return this.request(options, 'getHeatmap');
      }
      /**
       * Get video statistics
       * @returns A {@link BunnyCdnStream.VideoStatisticsResponse} instance.
       * @param data The data to fetch video statistics with
       * @example
       * ```typescript
       * await stream.getVideoStatistics("0273f24a-79d1-d0fe-97ca-b0e36bed31es")
       * ```
       */
      async getVideoStatistics(videoId, data = {}) {
          const options = this.getOptions();
          options.url += `/library/${this.options.videoLibrary}/statistics`;
          options.data = JSON.stringify({ ...data, videoGuid: videoId });
          return this.request(options, 'fetch');
      }
      /**
       * Force re-encode a video
       *
       * NOTE: This will not work if keepOriginal is set to false
       * @returns A {@link BunnyCdnStream.VideoResponse} instance.
       * @param videoId The video ID
       * @example
       * ```typescript
       * await stream.reencodeVideo("0273f24a-79d1-d0fe-97ca-b0e36bed31es")
       * ```
       */
      async reencodeVideo(videoId) {
          const options = this.getOptions();
          options.url += `/library/${this.options.videoLibrary}/videos/${videoId}/reencode`;
          options.method = 'POST';
          const video = await this.request(options, 'reencode');
          return new BunnyCdnStreamVideo(video);
      }
      /**
       * List videos
       * @returns An array of {@link BunnyCdnStream.VideoStatisticsResponse} instances.
       * @param data The options to list videos with
       * @example
       * ```typescript
       * await stream.listVideos({ page: 2, search: "The best title", itemsPerPage: 10 })
       * ```
       */
      async listVideos(data = {}) {
          const options = this.getOptions();
          options.url += `/library/${this.options.videoLibrary}/videos`;
          options.params = data;
          const videos = await this.request(options, 'list');
          return {
              ...videos,
              items: videos.items.map((video) => new BunnyCdnStreamVideo(video))
          };
      }
      /**
       * List all videos with an optional callback between each page
       * @returns An array of {@link BunnyCdnStream.VideoStatisticsResponse} instances.
       * @param data The options to list videos with
       * @param stop The callback that if returns ``true`` stops the iteration
       * @example
       * ```typescript
       * await stream.listAllVideos()
       * ```
       */
      async listAllVideos(data = {}, stop) {
          const all = [];
          let nextPage = true;
          let page = 1;
          while (nextPage) {
              const videos = await this.listVideos({
                  ...data,
                  page,
                  itemsPerPage: data.itemsPerPage || 100
              });
              const totalPages = Math.ceil(videos.totalItems / videos.itemsPerPage);
              all.push(...videos.items);
              if (stop && (await stop(videos.items, page, totalPages))) {
                  nextPage = false;
                  continue;
              }
              if (page < totalPages) {
                  page++;
              }
              else {
                  nextPage = false;
              }
          }
          return all;
      }
      /**
       * Set the thumbnail
       *
       * NOTE: The file type is automatically detected from the buffer, however if it fails, it will default to ``image/jpeg``
       * @returns A {@link BunnyCdnStream.SetThumbnailVideoResponse} instance.
       * @param videoId The video ID
       * @param thumbnail A buffer/stream/url of the thumbnail
       * @param overrideContentType The content type to override and skip the automatic detection
       * @example
       * ```typescript
       * await stream.setThumbnail("0273f24a-79d1-d0fe-97ca-b0e36bed31es", readFileSync("thumbnail.jpg"))
       * ```
       */
      async setThumbnail(videoId, thumbnail, overrideContentType) {
          const options = this.getOptions();
          const ct = overrideContentType ? { mime: overrideContentType } : undefined;
          if (typeof thumbnail !== 'string')
              options.headers['Content-Type'] =
                  thumbnail instanceof fs.ReadStream ? 'application/octet-stream' : (ct || (await fileType.fromBuffer(thumbnail)) || { mime: 'image/jpg' }).mime;
          options.url += `/library/${this.options.videoLibrary}/videos/${videoId}/thumbnail`;
          options.method = 'POST';
          if (typeof thumbnail === 'string') {
              options.params = { thumbnailUrl: thumbnail };
          }
          else {
              options.data = thumbnail;
          }
          return this.request(options, 'setThumbnail');
      }
      /**
       * Upload a video using a URL
       *
       * NOTE: This will not work if the video is not public, and the thumbnail/preview will not be regenerated for existing videos
       * @returns A {@link BunnyCdnStream.FetchVideoResponse} instance
       * @param videoId The video ID
       * @param data The data with video url to fetch and optional headers
       * @example
       * ```typescript
       * await stream.fetchVideo("0273f24a-79d1-d0fe-97ca-b0e36bed31es", { url: "https://example.com/file.mp4" })
       * ```
       */
      async fetchVideo(videoId, data) {
          const options = this.getOptions();
          options.url += `/library/${this.options.videoLibrary}/videos/${videoId}/fetch`;
          options.method = 'POST';
          options.data = JSON.stringify(data);
          return this.request(options, 'fetch');
      }
      /**
       * Add captions to a video
       * @returns A {@link BunnyCdnStream.AddCaptionsVideoResponse} instance.
       * @param videoId The video ID
       * @param data The data to add captions with where the captions file is a buffer or base64 string
       * @example
       * ```typescript
       * await stream.addCaptions("0273f24a-79d1-d0fe-97ca-b0e36bed31es", { captionsFile: readFile("./subtitles.srt"), label: "English", srclang: "en" })
       * ```
       */
      async addCaptions(videoId, data) {
          const options = this.getOptions();
          options.url += `/library/${this.options.videoLibrary}/videos/${videoId}/captions/${data.srclang}`;
          options.method = 'POST';
          if (typeof data.captionsFile !== 'string') {
              data.captionsFile = data.captionsFile.toString('base64');
          }
          options.data = JSON.stringify(data);
          return this.request(options, 'addCaptions');
      }
      /**
       * Delete captions from a video
       * @returns A {@link BunnyCdnStream.DeleteCaptionsVideoResponse} instance.
       * @param videoId The video ID
       * @param srclang The specified srclang used when creating
       * @example
       * ```typescript
       * await stream.deleteCaptions("0273f24a-79d1-d0fe-97ca-b0e36bed31es", "en")
       * ```
       */
      async deleteCaptions(videoId, srclang) {
          const options = this.getOptions();
          options.url += `/library/${this.options.videoLibrary}/videos/${videoId}/captions/${srclang}`;
          options.method = 'DELETE';
          return this.request(options, 'deleteCaptions');
      }
      /**
       * Create a collection
       * @returns A {@link BunnyCdnStream.CreateCollectionResponse} instance.
       * @param name The collection name
       * @example
       * ```typescript
       * await stream.createCollection("New Collection")
       * ```
       */
      async createCollection(name) {
          const options = this.getOptions();
          options.url += `/library/${this.options.videoLibrary}/collections`;
          options.method = 'POST';
          options.data = JSON.stringify({ name });
          return this.request(options, 'createCollection');
      }
      /**
       * Retrieve info about a collection from BunnyCdn
       * @returns A {@link BunnyCdnStream.BunnyCdnStreamCollection} instance.
       * @param collectionId The collection ID
       * @example
       * ```typescript
       * await stream.getCollection("0273f24a-79d1-d0fe-97ca-b0e36bed31es")
       * ```
       */
      async getCollection(collectionId) {
          const options = this.getOptions();
          options.url += `/library/${this.options.videoLibrary}/collections/${collectionId}`;
          return this.request(options, 'getCollection');
      }
      /**
       * List collections
       * @returns a {@link BunnyCdnStream.ListCollectionsResponse} instances.
       * @param data The options to list collections with
       * @example
       * ```typescript
       * await stream.listCollections({ page: 2, search: "Y collections", itemsPerPage: 100, orderBy: 'date' })
       * ```
       */
      async listCollections(data = {}) {
          const options = this.getOptions();
          options.url += `/library/${this.options.videoLibrary}/collections`;
          options.params = { ...data };
          const collections = await this.request(options, 'listCollections');
          return collections;
      }
      /**
       * List all collections with an optional callback between each page
       * @returns An array of {@link BunnyCdnStream.BunnyCdnStreamCollection} instances.
       * @param data The options to list collections with
       * @param stop The callback that if returns ``true`` stops the iteration
       * @example
       * ```typescript
       * await stream.listAllCollections()
       * ```
       */
      async listAllCollections(data = {}, stop) {
          const all = [];
          let nextPage = true;
          let page = 1;
          while (nextPage) {
              const collections = await this.listCollections({
                  ...data,
                  page,
                  itemsPerPage: data.itemsPerPage || 100
              });
              const totalPages = Math.ceil(collections.totalItems / collections.itemsPerPage);
              all.push(...collections.items);
              if (stop && (await stop(collections.items, page, totalPages))) {
                  nextPage = false;
                  continue;
              }
              if (page < totalPages) {
                  page++;
              }
              else {
                  nextPage = false;
              }
          }
          return all;
      }
      /**
       * Update info of a collection
       * @returns A {@link BunnyCdnStream.UpdateCollectionResponse} instance.
       * @param collectionId The collection ID
       * @example
       * ```typescript
       * await stream.updateCollection("0273f24a-79d1-d0fe-97ca-b0e36bed31es", { name: 'New Collection'})
       * ```
       */
      async updateCollection(collectionId, data) {
          const options = this.getOptions();
          options.url += `/library/${this.options.videoLibrary}/collections/${collectionId}`;
          options.method = 'POST';
          options.data = JSON.stringify(data);
          return this.request(options, 'updateCollection');
      }
      /**
       * Delete a collection
       * @returns A {@link BunnyCdnStream.DeleteCollectionResponse} instance.
       * @param collectionId The collection ID
       * @example
       * ```typescript
       * await stream.deleteCollection("0273f24a-79d1-d0fe-97ca-b0e36bed31es")
       * ```
       */
      async deleteCollection(collectionId) {
          const options = this.getOptions();
          options.url += `/library/${this.options.videoLibrary}/collections/${collectionId}`;
          options.method = 'DELETE';
          return this.request(options, 'deleteCollection');
      }
      /**
       * Delete all collections
       * @returns void
       * @example
       * ```typescript
       * await stream.deleteAllCollections()
       * ```
       */
      async deleteAllCollections() {
          const iterate = async (page) => {
              const { items: collections } = await this.listCollections({ page });
              if (collections.length === 0)
                  return;
              await Promise.all(collections.map((collection) => this.deleteCollection(collection.guid)));
              await iterate(page + 1);
          };
          await iterate(1);
      }
      /**
       * Generate a direct upload tus
       *
       * NOTE: metadata.filetype is required for the tus upload to work
       * @returns A {@link BunnyCdnStream.CreateDirectUpload}
       * @param data The data to create the video with
       * @param expirationTime The expiration time of the tus upload
       * @example
       * ```typescript
       * await stream.createDirectUpload({ title: "My Video" })
       * ```
       */
      async createDirectUpload(data, expirationDate) {
          // create a video
          const expirationTimestamp = (expirationDate || new Date(Date.now() + 60000)).getTime();
          const video = await this.createVideo(data);
          const hash = this.generateTUSHash(video.guid, expirationTimestamp);
          return {
              video,
              endpoint: 'https://video.bunnycdn.com/tusupload',
              headers: {
                  AuthorizationSignature: hash,
                  AuthorizationExpire: expirationTimestamp,
                  VideoId: video.guid,
                  LibraryId: this.options.videoLibrary
              },
              metadata: {
                  filetype: '',
                  title: data.title,
                  collection: data.collection
              }
          };
      }
      generateTUSHash(videoId, expirationTime) {
          // sha256(library_id + api_key + expiration_time + video_id)
          return node_crypto.createHash('sha256')
              .update(this.options.videoLibrary.toString() + this.options.apiKey.toString() + expirationTime.toString() + videoId.toString())
              .digest('hex');
      }
      async request(options, name) {
          try {
              const req = await axios.request(options);
              if (typeof req.data === 'object') {
                  req.data = lowerObject(req.data);
              }
              if ('message' in req.data &&
                  typeof req.data.message === 'string' &&
                  'statusCode' in req.data &&
                  typeof req.data.statusCode === 'number' &&
                  req.data.statusCode !== 200) {
                  throw new BunnyCdnStreamError(req.data.message, name, req.data.statusCode);
              }
              return req.data;
          }
          catch (error) {
              throw new BunnyCdnStreamError(error, name);
          }
      }
      getOptions() {
          const { videoLibrary, apiKey, ...options } = this.options;
          return {
              ...options,
              ...this.axiosOptions,
              headers: new axios.AxiosHeaders(this.axiosOptions.headers)
          };
      }
  }

  exports.BunnyCdnStream = BunnyCdnStream;
  exports.BunnyCdnStreamVideo = BunnyCdnStreamVideo;

}));
//# sourceMappingURL=index.umd.js.map
